use piston_window::*;
use piston_window::math::*;

static TURN_RATE: Scalar = 4.0;
static THRUST_RATE: Scalar = 4.0;

pub struct Controls {
    up_d: bool,
    down_d: bool,
    left_d: bool,
    right_d: bool,
    fire: bool
}

impl Controls {
    pub fn new() -> Self {
        Controls {
            up_d: false, down_d: false, left_d: false, right_d: false,
            fire: false
        }
    }

    pub fn update(&mut self, inp: Input) {
        match inp {
            Input::Press(b) => {
                match b {
                    Button::Keyboard(Key::Up)    => { self.up_d    = true; }
                    Button::Keyboard(Key::Down)  => { self.down_d  = true; }
                    Button::Keyboard(Key::Left)  => { self.left_d  = true; }
                    Button::Keyboard(Key::Right) => { self.right_d = true; }
                    _ => {}
                }
            }
            Input::Release(b) => {
                match b {
                    Button::Keyboard(Key::Up)    => { self.up_d    = false; }
                    Button::Keyboard(Key::Down)  => { self.down_d  = false; }
                    Button::Keyboard(Key::Left)  => { self.left_d  = false; }
                    Button::Keyboard(Key::Right) => { self.right_d = false; }
                    Button::Keyboard(Key::Space) => { self.fire    = true; }
                    _ => {}
                }
            }
            _ => {}
        }
    }

    pub fn velocity_change(&self, rot: Scalar, dt: Scalar) -> Vec2d {
        let thrust = if self.up_d {
            THRUST_RATE * dt
        } else if self.down_d {
            -THRUST_RATE * dt
        } else {
            0.0
        };
        if thrust != 0.0 {
            [rot.cos() * thrust, rot.sin() * thrust]
        } else {
            [0.0, 0.0]
        }
    }

    pub fn rotation_change(&self, dt: Scalar) -> Scalar {
        if self.left_d {
            -TURN_RATE * dt
        } else if self.right_d {
            TURN_RATE * dt
        } else {
            0.0
        }
    }

    pub fn fire_check(&mut self) -> bool {
        let firing = self.fire;
        self.fire = false;
        firing
    }
}
